# TGAM Pattern Library

This project is a living style guide. It uses Gulp tasks to generate a static website, which can then be deployed to a web server (upload the contents of the `public` directory). It also doubles as an NPM module. `npm install` it into your projects and you'll be able to import the CSS and JavaScript from the `dist` folder (see the "Importing the pattern library NPM module into outside projects" section, below).

## Project setup

1. If you haven't already done so, download and install [Node.js](https://nodejs.org/) (which includes the [NPM command-line tool](https://www.npmjs.com/)).
2. Download or clone this repository.
3. Inside your terminal, navigate to the project directory.
4. Run `npm install`.
5. Run `./node_modules/.bin/gulp`.
6. Visit `http://localhost:3000` in your web browser.

## Project overview

The codebase is divided into the following directories:

- `dist`: Contains a distribution-ready copy of the pattern library files. The contents of this directory will be published as an NPM module, which outside projects will then be able to install as a dependency.
- `gulp`: Contains all of the [Gulp](http://gulpjs.com/) tasks that are responsible for generating the static website and copying the pattern library files into the distribution folder.
- `public`: Contains the generated static website files. The contents of this directory should be deployed to a web server.
- `src`: Contains all of the "working files". You'll spend most of your time inside of this directory.
  - `patterns`: Contains the actual pattern library files (which will be copied into the `dist` directory).
  - `specimens`: Contains examples of patterns in action, which will serve as living documentation on the static website.
  - `site`: Files in this directory define the structure, behaviour and visual style of the static website.

## Workflow

The examples in the `specimens` directory serve as living documentation for how patterns in the library are intended to be used. During development, the `specimens` directory can be treated like a workspace where you can play around with existing patterns and prototype new ideas. Once you're happy with your creation, you should formalize your pattern by migrating the relevant CSS and JavaScript into the `patterns` directory and adding notes to your specimen to describe its purpose, etc.

To better understand the difference between `patterns` and `specimens`, you may find it helpful to think in terms of Object-Oriented Programming: The code in the `patterns` directory defines "classes", and the examples in the `specimens` directory demonstrate how to create instances of these "classes". External projects will import only the "classes", and it will be up to developers to "instantiate" them in their code, using the examples as a guide.

Start by creating a new directory inside of `specimens`, with the following (empty) files inside of it:

```
src
  specimens
    my-specimen
      data.json
      markup.hbs
      notes.html
      script.js
      styles.scss
```

**Note**: Each specimen lives inside of its own directory and each directory must have a unique name (even if it's nested inside of another directory). For example, in the following scenario, you would need to rename one of the `my-specimen` directories:

```
src
  specimens
    subdirectory-1
      my-specimen
    subdirectory-2
      my-specimen    <-- Bad!
```

During the initial prototyping phase, feel free to experiment by writing arbitrary HTML markup, CSS and JavaScript inside of the `markup.hbs`, `styles.scss` and `script.js` files. Once your pattern is ready for production, however, your code should adhere to the following rules:

### `markup.hbs` and `data.json`

The `markup.hbs` file should contain only the HTML markup that's required to create an "instance" of a pattern. Developers should be able to copy and paste this markup into their external projects and everything should "just work" (provided they've installed the pattern library NPM module and compiled the pattern library SCSS).

If you're planning to include the pattern inside of another pattern, then you should replace hard-coded, context-specific values (such as link text, form label text, etc.) with [Handlebars](http://handlebarsjs.com/) variables. For example:

```
<div class="user-name-field">
  <label for="user-name">Your name</label>
  <input id="user-name" type="text" placeholder="Enter your name">
</div>
```

Could become:

```
<div class="user-name-field">
  <label for="user-name">{{ label }}</label>
  <input id="user-name" type="text" placeholder="{{ placeholder }}">
</div>
```

You would then assign values to your variables, in the specimen's `data.json` file:

```
{
  "label": "Your name",
  "placeholder": "Enter your name"
}
```

Now, assuming your specimen files are inside of a directory named `src/specimens/user-name-field`, a different specimen should be able to include your specimen, like this:

**src/specimens/login-form/markup.hbs**

```
<div class="login">
  <h2>{{ heading }}</h2>
  <form>
    {{> user-name-field childData }}
    <input type="submit" value="{{ submit }}">
  </form>
</div>
```

**src/specimens/login-form/data.json**

```
{
  "heading": "Please login",
  "childData": {
    "label": "User name",
    "placeholder": "What's your user name?"
  },
  "submit": "Submit"
}
```

See the "Handlebars partials" section below, for more information.

### `notes.html`

The markup inside of this file will appear on the static website, alongside the `iframe` that display's your specimen's HTML. This file is optional.

### `script.js`

Although few patterns will require JavaScript, complex user interface elements like carousels and accordion menus may require click handlers to add/remove CSS classes, dispatch events, etc. The JavaScript that defines this behaviour should live inside of the pattern library (`src/patterns/tgam/javascripts`) and should be written in a modular way, using [CommonJS modules](https://webpack.github.io/docs/commonjs.html) or [ES2015 classes](https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Classes). For example:

**src/patterns/tgam/javascripts/carousel.js**

```
"use strict";

let element;

function init(domElement) {
  element = domElement;
  console.log("Initialized " + element);
}

module.exports = {
  init
};

```

**src/specimens/carousel/script.js**

```
"use strict";

let Carousel = require("tgam/javascripts/carousel.js");

let elements = document.getElementsByClassName("c-carousel");
if (elements) {
  for (let i = 0; i < elements.length; i++) {
    let instance = Object.create(Carousel);
    instance.init(elements[i]);
  }
}

```

`script.js` files should contain boilerplate code that demonstrates how to instantiate and use a pattern, not the pattern logic itself. Developers should be able to copy and paste this code into their external projects and everything should "just work" (provided they've installed the pattern library NPM module and transpiled/bundled the pattern library JavaScript).

### `styles.scss`

This file contains additional SCSS that your specimen requires for demonstration purposes, but not the pattern SCSS itself. Pattern SCSS should always live inside the pattern library (`src/patterns/tgam`).

`styles.scss` files could contain things like media query breakpoint values, but not the contents of the media query itself. We don't want to include any SCSS that directly governs the appearance of the pattern, only ancillary code. An explanation of the media query situation:

Because media queries are only aware of the viewport's dimensions and not the dimensions of individual elements, and our pattern library components could be nested inside of elements that are less wide than the viewport (i.e. sidebar columns, etc.), we won't be able to author reliable, width-based media queries in our pattern library SCSS. Someday, when [CSS element/container queries](https://www.filamentgroup.com/lab/element-query-workarounds.html) exist (or we decide to use a [JavaScript prolyfill](http://elementqueries.com/)), we'll be able to write truly encapsulated components. Until then, we'll have to define our patterns' breakpoints from higher-level, parent container contexts. For example:

**src/patterns/tgam/_objects.media.scss**

```
.o-media {
  display: flex;
  align-items: flex-start;
}
.o-media__body {
  flex: 1;
}
@mixin o-media-narrow {
  .o-media {
    display: block; // Stack elements in a single column
  }
}
```

**src/specimens/multi-column-layout/styles.scss**

```
@media all and (max-width: 650px) {
  .l-multi-column__left {
    @include o-media-narrow;
  }
}
```

Instead of defining a media query inside of the pattern SCSS itself, we provide a mixin that outside SCSS can `@include` when we need to put the pattern into the "narrow" state (since the pattern itself isn't capable of knowing when this needs to happen).

### Handlebars partials

For each specimen in the `src/specimens` directory, two [Handlebars](http://handlebarsjs.com/) partials will automatically be registered for you. For example, if your specimen lived inside of a directory named `src/specimens/fancy-sidebar`, then the following Handlebars partials would be available for you to use:

1. `{{> fancy-sidebar }}`
2. `{{> fancy-sidebar-snippet }}`

These are essentially just file includes ([read more about partials here](http://handlebarsjs.com/partials.html)).

The first one contains the specimen's HTML markup, and is intended to be used primarily inside of other specimens (i.e. a `site-header` specimen's markup may contain an `advertising-banner` specimen's markup).

The second one contains an HTML snippet that includes markup for an `iframe` and a collapsable information panel that's intended to be displayed on static site pages. This partial takes three optional arguments: `initialWidth`, `hideHtml` and `hideJs`. The `initialWidth` argument allows you to set the default width of the `iframe` and setting `hideHtml` or `hideJs` to `true` will prevent the corresponding content from appearing inside of the collapsable information panel. For example, you could set the width of the `fancy-sidebar` specimen's `iframe` and hide it's HTML like this:

`{{> fancy-sidebar-snippet initialWidth="400px" hideHtml=true }}`

See the "Creating pages" section below.

## Previewing your work

In order to preview your work, you'll need to compile the static website by running the Gulp task (`./node_modules/.bin/gulp`). Once it's running, Gulp should monitor your files for changes and recompile your SASS as required. If you encounter a situation where Gulp isn't recompiling properly, then just stop it via `control + c` (on your keyboard) and restart it via `./node_modules/.bin/gulp` (in your terminal). While Gulp is running, you'll be able to view the static website by navigating to http://localhost:3000 in your web browser.

Before you'll be able to view your newly-created pattern in your web browser, you'll need to add it to a page in the static website (see below).

## Creating pages

To add a new page to the static website, create a new Handlebars template (an HTML file with an `.hbs` extension) inside of the `src/site/pages/` directory. When the Gulp tasks run, the contents of this file will be inserted into the `<body>` of the generated page. You can use Handlebars partials to pull in specimen snippets, like this: `{{> my-specimen-snippet }}`, where `my-specimen` is the name of the specimen directory (see the "Workflow" section, above). Pages also contain a `pageConfig` HTML code comment, which you can use to set the page's title and meta title:

```
<!--pageConfig

metaTitle: Home page
pageTitle: Fancy website - Home

pageConfig-->
```

Pages that you create inside of `src/site/pages/` will automatically end up inside the root directory of the static website. For example:

- `src/site/pages/my-page.hbs` will become `my-page.html`
- `src/site/pages/cat-videos/index.hbs` will become `cat-videos/index.html`

## Adding links to the menu

To add a new link to the static website's navigation menu, edit the `src/site/navigation.yml` file. This file defines the structure of the menu, in [YAML](http://www.yaml.org/start.html) format. It should look something like this:

```
primaryNav:
  -
    name: "Home"
    path: "/"
  -
    name: "Another page"
    path: "/another-page.html"
  -
    name: "External link"
    path: "http://www.google.com"
  -
    name: "Some category"
    path: "/some-category"
    children:
      -
        name: "Nested page"
        path: "/some-category/nested-page"
```

The above menu structure should generate HTML similar to this:

```
<ul>
  <li><a href="/">Home</a></li>
  <li><a href="/another-page.html">Another page</a></li>
  <li><a href="http://www.google.com">External link</a></li>
  <li>
    <a href="/some-category">Some category</a>
    <ul>
      <li><a href="/some-category/nested-page">Nested page</a></li>
    </ul>
  </li>
</ul>
```

**Note**: only one level of link nesting is currently supported.

## Working with the pattern library files

The pattern library consists of the SASS, JavaScript, fonts and images that live inside of the `src/patterns/tgam` directory:

```
src/patterns/tgam
src/patterns/tgam/fonts
src/patterns/tgam/images
src/patterns/tgam/javascripts
```

These are the files that get copied into the `dist` directory when the Gulp tasks run, and get published as an NPM module when you run `npm publish` (see the "Publishing new versions of the pattern library NPM module" section, below).

----------
----------

**Aside**: We decided not to include HTML files in the pattern library because importing HTML snippets directly into our projects doesn't align with our needs. Here's a brief overview of the pros and cons of each approach:

#### Pattern HTML imported directly into projects

Rather than importing discrete chunks of HTML, we would create a "Content API" (a set of functions that return HTML markup). Various arguments would be passed into the Content API functions (i.e. class names, text strings, etc.), which would result in different markup being returned. Lonely Planet followed this strategy with their ["Rizzo" engine](http://rizzo.lonelyplanet.com/styleguide/ui-components/cards#styleguide-block__item--1).

**Pros**

- **Maintenance**: Easier to maintain because the HTML lives in a single place. Updates are automatically applied to all instances of the code, in every project that uses the pattern library.
- **Consistency**: Developers are not able to modify the code, which means that they aren't able to break things and they can't create a million subtle variations of a single pattern.

**Cons**

- **Frustration**: Because developers aren't able to modify the code directly, it's difficult for them to add things like `aria` and `data` attributes to individual instances of patterns. The only way for them to do this is to modify the Content API function that returns the markup (i.e. add a new argument to the function signature and write some additional logic inside the function body). Another alternative would be to "fork" the pattern by copying and pasting the HTML that the function returned and working with that HTML directly.

#### Pattern HTML written by hand

**Pros and cons**

- Developers are free to make subtle adjustments to pattern markup, but must remain disciplined when doing so. Care must be taken not to alter patterns too much, otherwise we could end up with a disparate array of mutated "Frankenpatterns" that are difficult to maintain.

----------
----------

## CSS methodology

The CSS in this project is organized according to the [ITCSS](http://itcss.io/) (Inverted Triangle CSS) methodology. If you aren't already familiar with ITCSS, then please [watch this video](https://www.youtube.com/watch?v=1OKZOV-iLj4).

```
\ ------------------------------------ /
 \              Settings              /
  \ -------------------------------- /
   \             Tools              /
    \ ---------------------------- /
     \          Generic           /
      \ ------------------------ /
       \          Base          /
        \ -------------------- /
         \      Objects       /
          \ ---------------- /
           \   Components   /
            \ ------------ /
             \   Trumps   /
              \ -------- /
```

### Import order

1. **Requirements** -- Vendor libraries
2. **Settings** ------ Global variables, site-wide settings, config switches, etc.
3. **Tools** --------- Site-wide mixins and functions.
4. **Generic** ------- "Ground-zero" styles. Low-specificity, far-reaching rulesets (e.g. normalize, resets, box-sizing).
5. **Base** ---------- Unclassed HTML elements (type selectors: `a {}`, `blockquote {}`, `address {}`, etc.)
6. **Objects** ------- Objects, abstractions, and "cosmetic-free" design patterns (e.g. `.media {}`).
7. **Components** ---- Designed components. Discrete, complete chunks of UI (e.g. `.carousel {}`).
8. **Trumps** -------- High-specificity, very explicit selectors. Overrides and helper classes (e.g. `.hidden {}`).

### Difference between objects and components

Objects provide the structure and Components provide the "skin".
Objects are like abstract classes and Components are like concrete classes.

### Resources

You may find it helpful to refer to the following resources for ideas on how to organize your CSS:

- [InuitCSS documentation](https://github.com/inuitcss/getting-started#import-order)
- [InuitCSS repository](https://github.com/inuitcss)
- [Files from Harry Roberts' .Net Magazine screencast](https://github.com/itcss/itcss-netmag)
- [Files from one of Harry Roberts' projects](https://github.com/csswizardry/frcss)

## Publishing new versions of the pattern library NPM module

1. Run `./node_modules/.bin/gulp` to generate the output files.
2. Commit your changes: `git commit -m "Your commit message"`.
3. Update the module version number: `npm version patch` or `npm version minor` or `npm version major` (this will create a new commit).
4. Re-publish the module: `npm publish`.
5. Push your commits: `git push`.

Refer to the [NPM documentation](https://docs.npmjs.com/getting-started/publishing-npm-packages) for more information.

The NPM module will be published to: https://www.npmjs.org/

## Importing the pattern library NPM module into outside projects

1. If you haven't already done so, install a task runner such as [Gulp](http://gulpjs.com/) (recommended) or [Grunt](http://gruntjs.com/).
2. If you haven't already done so, initialize your project via `npm init`.
3. Install the pattern library module via `npm install --save-dev tgam-patterns`.
4. Install the `node-sass-asset-functions` module via `npm install --save-dev node-sass-asset-functions`. This module will allow us to process the `font-url()` and `image-url()` SASS helpers that the pattern library uses.
5. Create a file called `gulpfile.js` (or edit your existing `gulpfile.js`) and add the following:

```
var gulp = require("gulp");
var sass = require("gulp-sass");
var assetFunctions = require("node-sass-asset-functions");
var tgamIncludePaths = require("tgam-patterns").includePaths;

// Copy font and image files from node_modules into a deployable directory

gulp.task("tgam-patterns:fonts", function() {
  return gulp.src("./node_modules/tgam-patterns/dist/tgam/fonts/**/*.*")
    .pipe(gulp.dest("./path/to/your/fonts"));
});

gulp.task("tgam-patterns:images", function() {
  return gulp.src("./node_modules/tgam-patterns/dist/tgam/images/**/*.*")
    .pipe(gulp.dest("./path/to/your/images"));
});

gulp.task("tgam-patterns", ["tgam-patterns:fonts", "tgam-patterns:images"], function(done) {
  done();
});

// Compile SASS files

gulp.task("styles:compile", function() {
  return gulp.src("./path/to/your/main/stylesheet.scss")
    .pipe(sass({
      includePaths: tgamIncludePaths, // SASS include paths
      functions: assetFunctions({
        // Specify base URLs for SASS helpers (font-url(), image-url(), etc.)
        images_path: "/path/to/your/images",
        fonts_path: "/path/to/your/fonts",
        http_images_path: "/path/to/your/images",
        http_fonts_path: "/path/to/your/fonts"
      })
    })
    .pipe(gulp.dest("./path/to/your/output/directory"));
});
```

You should now be able to import patterns into your SCSS files via `@import "tgam/all";` and compile them via `gulp styles:compile`. You'll also need to run `gulp tgam-patterns` after installing the pattern library module.

### Updating the NPM module

1. Run `npm outdated` in your command line to see if a newer version of the pattern library is available.
2. Run `npm update tgam-patterns`.
3. Run `gulp tgam-patterns` to copy the font and image assets.

## TODO's

- Add more robust error handling.
- Add eslint and sass-lint (with configs that match what we're using on tgam.io).
